;;; GNU Guix --- Functional package management for GNU
;;; Copyright © 2016 Roel Janssen <roel@gnu.org>
;;;
;;; This file is part of GNU Guix.
;;;
;;; GNU Guix is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 3 of the License, or (at
;;; your option) any later version.
;;;
;;; GNU Guix is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.

(define-module (guix scripts workflow)
  #:use-module (guix ui)
  #:use-module (guix scripts)
  #:use-module (guix workflows)
  #:use-module (srfi srfi-37)
  #:export (guix-workflow))

(define (show-help)
  (display (_ "Usage: guix workflow [OPTION]...
Run programs in a pre-defined sequence.\n"))
  (display (_ "
  -i, --input LOCATION ...
                         set LOCATION as input for a workflow"))
  (display (_ "
  -o, --output LOCATION ...
                         set LOCATION as output for a workflow"))
  (display (_ "
  -h, --help
                         display this help and exit"))
  (display (_ "
  -V, --version          display version information and exit"))
  (newline))

(define %input-parameter #f)
(define %output-parameter #f)
(define %workflow-exp #f)

(define (set-parameter args)
  (let ((input-arg (list-ref (option-names (list-ref args 0)) 0))
        (input-opt (list-ref args 2)))
    (cond ((eq? input-arg #\i) (set! %input-parameter input-opt))
          ((eq? input-arg #\o) (set! %output-parameter input-opt))
          ((eq? input-arg #\e) (set! %workflow-exp input-opt))
          ((#t (format #t "Invalid option ~a" input-arg))))))

(define %options
  ;; List of command-line options.
  (cons* (list (option '(#\h "help") #f #f
                       (lambda args
                         (show-help)
                         (exit 0)))
               (option '(#\V "version") #f #f
                       (lambda args
                         (show-version-and-exit "guix workflow")))
               (option '(#\e "execute") #t #f
                       (lambda args
                         (set-parameter args)))
               (option '(#\i "input") #t #f
                       (lambda args
                         (set-parameter args)))
               (option '(#\o "output") #t #f
                       (lambda args
                         (set-parameter args))))))

;;;
;;; Entry point.
;;;

(define (guix-workflow . args)
  (let ((opts (parse-command-line args %options '())))
    (if (or (eq? %output-parameter #f)
            (eq? %input-parameter #f)
            (eq? %workflow-exp #f))
        (display "Please provide --input, --output and --execute arguments.")
        (display "Workflow execution hasn't been implemented yet."))
    (newline)))
