(define-module (guix workflows execution-helpers)
  #:use-module (guix utils)
  #:use-module (guix build utils)
  #:use-module (guix workflows utils)

  #:export (interpreter-specific-action
            generate-script-header
            interpreter-specific-mkdir))

(define (interpreter-specific-action shell
                                     guile-action
                                     shell-action
                                     python-action
                                     perl-action
                                     other-action)
  (cond
     ((equal? shell 'guile)
      (guile-action))
     ((or (equal? shell 'sh) (equal? shell 'bash))
      (shell-action))
     ((equal? shell 'python)
      (python-action))
     ((equal? shell 'perl)
      (perl-action))
     (#t "")))

(define (generate-script-header interpreter)
  (interpreter-specific-action interpreter
  (lambda _ "#!/usr/bin/guile --no-auto-compile\n-*- scheme -*-\n!#") ; Guile
  (lambda _ "#!/bin/sh") ; Shell
  (lambda _ "#!/usr/bin/python") ; Python
  (lambda _ "#!/usr/bin/perl") ; Perl
  "")) ; Other

(define (interpreter-specific-mkdir interpreter directory)
  (interpreter-specific-action interpreter
     ;; Guile
     (lambda () (format #f "~s" `(unless (access? ,directory F_OK)
                                   (mkdir ,directory))))
     ;; Shell
     (lambda () (string-append "mkdir -p " directory))
     ;; Python
     (lambda () (string-append "import os\n"
                               "if not os.path.exists(\""
                               directory
                               "\"): os.makedirs(\""
                               directory
                               "\")"))
     ;; Perl
     (lambda () (string-append "use File::Path;\n"
                               "make_path('"
                               directory
                               "');"))
     ;; Other
     ""))
